<?php
namespace Test;

use ByJG\DbMigration\Migration;
use ByJG\Util\Uri;

class MigrationTest extends \PHPUnit\Framework\TestCase
{
    /**
     * @var Migration
     */
    protected $object;

    public function setUp()
    {
        $this->object = new Migration(new Uri('mysql://localhost'), __DIR__ . '/dirstructure');
    }

    public function tearDown()
    {
        $this->object = null;
    }

    public function testGetBaseSql()
    {
        $base = $this->object->getBaseSql();
        $this->assertEquals(__DIR__ . '/dirstructure/base.sql', $base);
    }

    /**
     * @expectedException \ByJG\DbMigration\Exception\InvalidMigrationFile
     */
    public function testGetBaseSqlNotFound()
    {
        $this->object = new Migration(new Uri('mysql://localhost'), __DIR__ . '/invalid');
        $this->object->getBaseSql();
    }

    public function testGetBaseSqlNotFoundAndNotRequired()
    {
        $this->object = new Migration(new Uri('mysql://localhost'), __DIR__ . '/invalid', false);
        $this->object->getBaseSql();
        $this->assertTrue(true);
    }

    public function testGetMigrationSql1()
    {
        $version = $this->object->getMigrationSql(1, 1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/up/00001.sql', $version);
    }

    public function testGetMigrationSql2()
    {
        $version = $this->object->getMigrationSql(2, 1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/up/00002.sql', $version);
    }

    public function testGetMigrationSql3()
    {
        $version = $this->object->getMigrationSql(12, 1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/up/00012-dev.sql', $version);
    }

    /**
     * @expectedException \ByJG\DbMigration\Exception\InvalidMigrationFile
     * @expectedExceptionMessage version number '13'
     */
    public function testGetMigrationSql4()
    {
        $this->object->getMigrationSql(13, 1);
    }

    public function testGetMigrationSqlDown1()
    {
        $version = $this->object->getMigrationSql(1, -1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/down/00001.sql', $version);
    }

    public function testGetMigrationSqlDown2()
    {
        $version = $this->object->getMigrationSql(2, -1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/down/00002.sql', $version);
    }

    public function testGetMigrationSqlDown3()
    {
        $version = $this->object->getMigrationSql(12, -1);
        $this->assertEquals(__DIR__ . '/dirstructure/migrations/down/00012-dev.sql', $version);
    }

    /**
     * @expectedException \ByJG\DbMigration\Exception\InvalidMigrationFile
     * @expectedExceptionMessage version number '13'
     */
    public function testGetMigrationSqlDown4()
    {
        $this->object->getMigrationSql(13, -1);
    }
}
